/////////////////////////////////////////////////////////////////
//Vector.h
//A collection of basic functions for dealing with vectors
//
//The Vector class contains three vector values
//The Vector class is normalized, and contains one scalar value
//
//The '*' operator has been overloaded to return the cross product
//////////////////////////////////////////////////////////////////

#ifndef _VECTOR_H
#define _VECTOR_H

#include <math.h>


/////////////////////////////////////////////////////////////////////
//Vector has x, y, and z variables, where scalar force doesn't always
//equal 1

class Vector
{
public:
	////////////////////////////////////////////////////
	//Constructors
	Vector();
	Vector(float set_x, float set_y, float set_z);
	Vector(float set_x, float set_y, float set_z, float set_F);
	
	////////////////////////////////////////////////////
	//Variables
	float x, y, z; //Vectors
	float F;
	
	////////////////////////////////////////////////////
	//Mathematic functions
	Vector normal() const; //Normalize
	Vector denorm() const;
		
	////////////////////////////////////////////////////
	//Accessors
	int is_norm() const;
	void set_norm(int val);

	////////////////////////////////////////////////////
	//Operators
	Vector operator+ (Vector rhs) const;	//Addition
	Vector operator- (Vector rhs) const;	//Subtraction
	Vector operator* (Vector rhs) const;	//Cross Product
	Vector operator* (float rhs) const;

	void operator+= (Vector rhs);
	void operator-= (Vector rhs);
	void operator*= (Vector rhs);
	
	bool operator== (Vector rhs);


private:
	bool is_normal;
};

Vector cross(Vector u, Vector v); //Prototype declarations
float dot(Vector u, Vector v);

Vector::Vector()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
	F = 0.0f;
	is_normal = false;
}

Vector::Vector(float set_x, float set_y, float set_z)
{
	is_normal = false;
	F = 0;

	x = set_x;
	y = set_y;
	z = set_z;
}

Vector::Vector(float set_x, float set_y, float set_z, float set_F)
{
	is_normal = true;
	
	x = set_x;
	y = set_y;
	z = set_z;
	F = set_F;
}

	

///////////////////////////////////////////////////////////////////
Vector Vector::normal() const
{
	if((*this).is_norm()) return (*this); //Trying to convert an already normal vector

	Vector temp;
	float tempf = powf(x,2.0f) + powf(y,2.0f) + powf(z, 2.0f);

	temp.F = sqrtf(tempf);
	temp.x = x/temp.F;
	temp.y = y/temp.F;
	temp.z = z/temp.F;
	temp.set_norm(true);
	
	return temp;
}

Vector Vector::denorm() const
{
	if(!(*this).is_norm()) return (*this);

	Vector temp;

	temp.x = x * F;
	temp.y = y * F;
	temp.z = z * F;
	temp.F = 0;
	temp.set_norm(false);

	return temp;
}


//////////////////////////////////////////////////////////////
int Vector::is_norm() const
{
	if (is_normal == false) return 0;
	else if (is_normal == true) return 1;
	else return -1;
}

void Vector::set_norm(int val)
{
	if (val == false) is_normal = false;
	else is_normal = true;
}


//////////////////////////////////////////////////////////////
Vector Vector::operator+ (Vector rhs) const
{
	Vector ret;

	if(rhs.is_norm()) rhs = rhs.denorm();
	
	if((*this).is_norm())
	{
		Vector temp = (*this).denorm();
		
		ret.x = temp.x + rhs.x;
		ret.y = temp.y + rhs.y;
		ret.z = temp.z + rhs.z;
		ret = ret.normal();
	}
	else
	{
		ret.x = x + rhs.x;
		ret.y = y + rhs.y;
		ret.z = z + rhs.z;
	}

	return ret;
}

Vector Vector::operator- (Vector rhs) const
{
	Vector ret;

	if(rhs.is_norm()) rhs = rhs.denorm();
	
	if((*this).is_norm())
	{
		Vector temp = (*this).denorm();
		
		ret.x = temp.x - rhs.x;
		ret.y = temp.y - rhs.y;
		ret.z = temp.z - rhs.z;
		ret = ret.normal();
	}
	else
	{
		ret.x = x - rhs.x;
		ret.y = y - rhs.y;
		ret.z = z - rhs.z;
	}

	return ret;
}

Vector Vector::operator* (Vector rhs) const
{
	if (rhs.is_norm()) rhs = rhs.denorm();

	if ((*this).is_norm())
	{
		Vector temp = (*this).denorm();
		temp = cross(temp, rhs);

		return temp.normal();
	}
	else return cross((*this), rhs);
}

Vector Vector::operator * (float rhs) const
{
	Vector temp;

	temp.x = x * rhs;
	temp.y = y * rhs;
	temp.z = z * rhs;

	return temp;
}

bool Vector::operator == (Vector rhs)
{
	if (x == rhs.x && y == rhs.y && z == rhs.z) return true;
	return false;
}


//////////////////
void Vector::operator += (Vector rhs)
{
	(*this) = (*this) + rhs;
}

void Vector::operator -= (Vector rhs)
{
	(*this) = (*this) - rhs;
}

void Vector::operator *= (Vector rhs)
{
	(*this) = (*this) * rhs;
}



///////////////////////////////////////////////////////////
//Vector related functions

Vector cross(Vector u, Vector v)
{
	Vector temp;

	temp.x = (u.y * v.z) - (u.z * v.y);
	temp.y = (u.z * v.x) - (u.x * v.z);
	temp.z = (u.x * v.y) - (u.y * v.x);
	 
	return temp;
}

//A dot product has different applications for both normalized
//and unnormalized vectors.  Thus there is no checking whether
//a vector is one or the other
float dot(Vector u, Vector v)
{
	return ((u.x * v.x) + (u.y * v.y) + (u.z * v.z));
}

float dist_squared(Vector u, Vector v)
{
	return ((u.x-v.x)*(u.x-v.x))+((u.y-v.y)*(u.y-v.y))+((u.z-v.z)*(u.z-v.z));
}

float length_squared(Vector u)
{
	return ((u.x*u.x)+(u.y*u.y)+(u.z*u.z));
}


#endif